﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text.Json.Serialization;
using Gallery.Illust;
using Xamarin.Forms;

namespace Gallery.Utils
{
    public class IllustResponse<T>
    {
        [JsonInclude]
        public bool error;
        [JsonInclude]
        public string message;
        [JsonInclude]
        public T body;
    }

    public class BookmarkResultData
    {
        [JsonInclude]
        public string last_bookmark_id;
        [JsonInclude]
        public string stacc_status_id;
    }

    public class Illust
    {
        [JsonInclude]
        public string illustId;
        [JsonInclude]
        public string illustTitle;
        [JsonInclude]
        public string id;
        [JsonInclude]
        public string title;
        [JsonInclude]
        public int illustType;
        [JsonInclude]
        public int xRestrict;
        [JsonInclude]
        public string url;
        [JsonInclude]
        public string description;
        [JsonInclude]
        public string[] tags;
        [JsonInclude]
        public string userId;
        [JsonInclude]
        public string userName;
        [JsonInclude]
        public int width;
        [JsonInclude]
        public int height;
        [JsonInclude]
        public int pageCount;
        [JsonInclude]
        public IllustBookmark bookmarkData;
        [JsonInclude]
        public string alt;
        [JsonInclude]
        public IllustUrls urls;
        [JsonInclude]
        public string seriesId;
        [JsonInclude]
        public string seriesTitle;
        [JsonInclude]
        public string profileImageUrl;

        public class IllustUrls
        {
            [JsonInclude]
            [JsonPropertyName("250x250")]
            public string x250;
            [JsonInclude]
            [JsonPropertyName("360x360")]
            public string x360;
            [JsonInclude]
            [JsonPropertyName("540x540")]
            public string x540;
        }

        public class IllustBookmark
        {
            [JsonInclude]
            public string id;
            [JsonInclude]
            [JsonPropertyName("private")]
            public bool isPrivate;
        }

        public IllustItem ConvertToItem(ImageSource image = null)
        {
            return new IllustItem
            {
                Id = illustId ?? id,
                BookmarkId = bookmarkData?.id,
                Title = illustTitle ?? title,
                IllustType = (IllustType)illustType,
                Image = image,
                ImageUrl = urls?.x360 ?? url,
                IsRestrict = xRestrict == 1,
                Tags = tags ?? new string[0],
                ProfileUrl = profileImageUrl,
                UserId = userId,
                UserName = userName,
                Width = width,
                Height = height,
                PageCount = pageCount
            };
        }
    }

    public class User
    {
        [JsonInclude]
        public string userId;
        [JsonInclude]
        public string name;
        [JsonInclude]
        public string image;
        [JsonInclude]
        public string imageBig;
        [JsonInclude]
        public bool premium;
        [JsonInclude]
        public bool isFollowed;
        //public string background;
        [JsonInclude]
        public int partial;
    }

    public class IllustFavoriteData : IllustResponse<IllustFavoriteBody> { }
    public class IllustFavoriteBody
    {
        [JsonInclude]
        public int total;
        [JsonInclude]
        public Illust[] works;
    }

    public class IllustData : IllustResponse<IllustBody> { }
    public class IllustBody
    {
        [JsonInclude]
        public Page page;
        [JsonInclude]
        public Thumbnail thumbnails;
        [JsonInclude]
        public User[] users;

        public class Page
        {
            [JsonInclude]
            public int[] follow;
            [JsonInclude]
            public Recommends recommend;
            [JsonInclude]
            public RecommendByTag[] recommendByTags;
            [JsonInclude]
            public Ranking ranking;
            [JsonInclude]
            public RecommendUser[] recommendUser;
            [JsonInclude]
            public EditorRecommend[] editorRecommend;
            [JsonInclude]
            public string[] newPost;

            public class Recommends
            {
                [JsonInclude]
                public string[] ids;
            }

            public class RecommendByTag
            {
                [JsonInclude]
                public string tag;
                [JsonInclude]
                public string[] ids;
            }

            public class Ranking
            {
                [JsonInclude]
                public RankingItem[] items;
                [JsonInclude]
                public string date;

                public class RankingItem
                {
                    [JsonInclude]
                    public string rank;
                    [JsonInclude]
                    public string id;
                }
            }

            public class RecommendUser
            {
                [JsonInclude]
                public int id;
                [JsonInclude]
                public string[] illustIds;
            }

            public class EditorRecommend
            {
                [JsonInclude]
                public string illustId;
                [JsonInclude]
                public string comment;
            }
        }

        public class Thumbnail
        {
            [JsonInclude]
            public Illust[] illust;
        }
    }

    public class IllustPreloadBody
    {
        [JsonInclude]
        public Dictionary<string, Illust> illust;
        [JsonInclude]
        public Dictionary<string, User> user;

        public class Illust
        {
            [JsonInclude]
            public string illustId;
            [JsonInclude]
            public string illustTitle;
            [JsonInclude]
            public string illustComment;
            [JsonInclude]
            public string id;
            [JsonInclude]
            public string title;
            [JsonInclude]
            public string description;
            [JsonInclude]
            public int illustType;
            [JsonInclude]
            public DateTime createDate;
            [JsonInclude]
            public DateTime uploadDate;
            [JsonInclude]
            public int xRestrict;
            [JsonInclude]
            public IllustUrls urls;
            [JsonInclude]
            public IllustTag tags;
            [JsonInclude]
            public string alt;
            [JsonInclude]
            public string userId;
            [JsonInclude]
            public string userName;
            [JsonInclude]
            public string userAccount;
            //public Dictionary<string, Illust> userIllusts;
            [JsonInclude]
            public int width;
            [JsonInclude]
            public int height;
            [JsonInclude]
            public int pageCount;
            [JsonInclude]
            public int bookmarkCount;
            [JsonInclude]
            public int likeCount;
            [JsonInclude]
            public int commentCount;
            [JsonInclude]
            public int responseCount;
            [JsonInclude]
            public int viewCount;
            [JsonInclude]
            public bool isOriginal;
            [JsonInclude]
            public IllustBookmark bookmarkData;

            public IllustItem CopyToItem(IllustItem item)
            {
                item.BookmarkId = bookmarkData?.id;
                item.Title = illustTitle ?? title;
                item.IllustType = (IllustType)illustType;
                item.ImageUrl = urls?.regular;
                item.IsRestrict = xRestrict == 1;
                if (tags != null && tags.tags != null)
                {
                    item.Tags = tags.tags.Where(t => t.locked).Select(t => t.tag).ToArray();
                }
                else
                {
                    item.Tags = new string[0];
                }
                item.UserId = userId;
                item.UserName = userName;
                item.Width = width;
                item.Height = height;
                item.PageCount = pageCount;
                return item;
            }

            [JsonIgnore]
            public string Url => urls.regular;

            public class IllustBookmark
            {
                [JsonInclude]
                public string id;
                [JsonInclude]
                [JsonPropertyName("private")]
                public bool isPrivate;
            }

            public class IllustUrls
            {
                [JsonInclude]
                public string mini;
                [JsonInclude]
                public string thumb;
                [JsonInclude]
                public string small;
                [JsonInclude]
                public string regular;
                [JsonInclude]
                public string original;
            }

            public class IllustTag
            {
                [JsonInclude]
                public string authorId;
                [JsonInclude]
                public bool isLocked;
                [JsonInclude]
                public IllustTagItem[] tags;
                [JsonInclude]
                public bool writable;

                public class IllustTagItem
                {
                    [JsonInclude]
                    public string tag;
                    [JsonInclude]
                    public bool locked;
                    [JsonInclude]
                    public bool deletable;
                    [JsonInclude]
                    public string userId;
                    [JsonInclude]
                    public IllustTranslate translation;
                    [JsonInclude]
                    public string userName;

                    public class IllustTranslate
                    {
                        [JsonInclude]
                        public string en;
                    }
                }
            }
        }
    }

    public class IllustPageData : IllustResponse<IllustPageBody[]> { }
    public class IllustPageBody
    {
        [JsonInclude]
        public Urls urls;
        [JsonInclude]
        public int width;
        [JsonInclude]
        public int height;

        public class Urls
        {
            [JsonInclude]
            public string thumb_mini;
            [JsonInclude]
            public string small;
            [JsonInclude]
            public string regular;
            [JsonInclude]
            public string original;
        }
    }

    public class IllustRecommendsData : IllustResponse<IllustRecommendsBody> { }
    public class IllustRecommendsBody
    {
        [JsonInclude]
        public Illust[] illusts;
        [JsonInclude]
        public string[] nextIds;
    }

    public class IllustUserListData : IllustResponse<IllustUserListBody> { }
    public class IllustUserListBody
    {
        [JsonInclude]
        public Dictionary<string, object> illusts;
    }

    public class IllustUserData : IllustResponse<IllustUserBody> { }
    public class IllustUserBody
    {
        [JsonInclude]
        public Dictionary<string, Illust> works;
    }

    public class IllustUgoiraData : IllustResponse<IllustUgoiraBody> { }
    public class IllustUgoiraBody
    {
        [JsonInclude]
        public string src;
        [JsonInclude]
        public string originalSrc;
        [JsonInclude]
        public string mime_type;
        [JsonInclude]
        public Frame[] frames;

        public class Frame
        {
            [JsonInclude]
            public string file;
            [JsonInclude]
            public int delay;

            public string FilePath;
            public bool Incompleted;
            public int First;
            public int Last;
            public int Offset;
            public int Length;
        }
    }
}
